@@ -4,10 +4,16 @@ from .telnet import Telnet
 
 from .datastream import AID
 
+from .attributes import (
+    Highlight,
+    Color
+)
+
 from .emulator import (
     Emulator,
     AttributeCell,
     CharacterCell,
+    CharacterSet,
     OperatorError,
     ProtectedCellOperatorError,
     FieldOverflowOperatorError
