@@ -11,7 +11,7 @@ import logging
 from .datastream import Command, WCC, Order, AID, parse_outbound_message, \
                         format_inbound_read_buffer_message, \
                         format_inbound_read_modified_message, \
-                        format_inbound_structured_fields
+                        parse_orders, format_inbound_structured_fields
 from .attributes import Attribute, AllExtendedAttribute, \
                         Highlight, HighlightExtendedAttribute, \
                         Color, ForegroundColorExtendedAttribute
@@ -268,12 +268,12 @@ class Emulator:
 
     def delete(self):
         """Delete key."""
+        if not self.is_formatted():
+            return
+
         if isinstance(self.cells[self.cursor_address], AttributeCell):
             raise ProtectedCellOperatorError
 
-        if not self.is_formatted():
-            raise NotImplementedError('Delete on unformatted screen is not supported')
-
         (_, end_address, attribute) = self.get_field(self.cursor_address)
 
         self._shift_left(self.cursor_address, end_address)
@@ -283,34 +283,32 @@ class Emulator:
 
     def erase_end_of_field(self):
         """Erase end of field (EOF) key."""
-        if isinstance(self.cells[self.cursor_address], AttributeCell):
-            raise ProtectedCellOperatorError
+        if self.is_formatted():
+            if isinstance(self.cells[self.cursor_address], AttributeCell):
+                raise ProtectedCellOperatorError
 
-        if not self.is_formatted():
-            raise NotImplementedError('Erase end of field on unformatted screen is not supported')
+            (_, end_address, attribute) = self.get_field(self.cursor_address)
 
-        (_, end_address, attribute) = self.get_field(self.cursor_address)
+            for address in self._get_addresses(self.cursor_address, end_address):
+                self._write_character(address, 0x00, preserve=True)
 
-        for address in self._get_addresses(self.cursor_address, end_address):
-            self._write_character(address, 0x00, preserve=True)
-
-        attribute.modified = True
+            attribute.modified = True
+        else:
+            for address in self._get_addresses(self.cursor_address, (self.rows * self.columns) - 1):
+                self._write_character(address, 0x00, preserve=True)
 
     def erase_input(self):
         """Erase input key."""
-        if not self.is_formatted():
-            raise NotImplementedError('Erase input on unformatted screen is not supported')
+        if self.is_formatted():
+            for (start_address, end_address, attribute) in self.get_fields():
+                for address in self._get_addresses(start_address, end_address):
+                    self._write_character(address, 0x00, preserve=True)
 
-        for (start_address, end_address, attribute) in self.get_fields():
-            for address in self._get_addresses(start_address, end_address):
-                self._write_character(address, 0x00, preserve=True)
+                attribute.modified = False
 
-            attribute.modified = False
-
-        # TODO: Confirm behavior but I think this should reposition the cursor to the
-        # first character location, after the field attribute, in the first unprotected
-        # field of the partition's character buffer - it is the same as Erase All
-        # Unprotected.
+            self.cursor_address = self._calculate_tab_address(0, 1) or 0
+        else:
+            self._clear()
 
     def get_bytes(self, start_address, end_address):
         """Get character cell bytes."""
@@ -413,18 +411,20 @@ class Emulator:
     def _erase_all_unprotected(self):
         self.logger.debug('Erase All Unprotected')
 
-        for (start_address, end_address, attribute) in self.get_fields():
-            for address in self._get_addresses(start_address, end_address):
-                self._write_character(address, 0x00, preserve=True)
+        if self.is_formatted():
+            for (start_address, end_address, attribute) in self.get_fields():
+                for address in self._get_addresses(start_address, end_address):
+                    self._write_character(address, 0x00, preserve=True)
 
-            attribute.modified = False
+                attribute.modified = False
+
+            self.cursor_address = self._calculate_tab_address(0, 1) or 0
+        else:
+            self._clear()
 
         self.current_aid = AID.NONE
         self.keyboard_locked = False
 
-        # TODO: Repositions the cursor to the first character location, after the field
-        # attribute, in the first unprotected field of the partition's character buffer.
-
     def _write(self, wcc, orders):
         if self.logger.isEnabledFor(logging.DEBUG):
             self.logger.debug('Write')
@@ -493,7 +493,23 @@ class Emulator:
 
                 self.address = self._wrap_address(self.address + 1)
             elif order == Order.MF:
-                raise NotImplementedError('MF order is not supported')
+                if isinstance(self.cells[self.address], AttributeCell):
+                    (attribute, extended_attributes) = data
+
+                    if attribute is not None:
+                        self.cells[self.address].attribute = attribute
+
+                    if extended_attributes:
+                        existing_formatting = self.cells[self.address].formatting
+
+                        # TODO: Confirm that this should affect "global" formatting...
+                        field_formatting = CellFormatting(existing_formatting, extended_attributes)
+
+                        self.cells[self.address].formatting = field_formatting
+
+                    self.address = self._wrap_address(self.address + 1)
+                else:
+                    self.logger.warning('MF order rejected as cell is not attribute cell')
             elif order == Order.RA:
                 (stop_address, byte, is_ge) = data
 
@@ -763,6 +779,8 @@ class Emulator:
         for (id_, data) in structured_fields:
             if id_ == StructuredField.READ_PARTITION:
                 self._read_partition(data)
+            elif id_ == StructuredField.ERASE_RESET:
+                self._erase(data[0] == 0x80)
             elif id_ == StructuredField.OUTBOUND_3270DS:
                 self._outbound_3270ds(data)
             else:
@@ -808,10 +826,10 @@ class Emulator:
             self.logger.warning(f'Partition 0x{partition:02x} not supported')
 
         if command == 0xf1:
-            self._write(WCC(data[2]), data[3:])
+            self._write(WCC(data[2]), parse_orders(data[3:]))
         elif command in [0xf5, 0x7e]:
             self._erase(command == 0x7e)
-            self._write(WCC(data[2]), data[3:])
+            self._write(WCC(data[2]), parse_orders(data[3:]))
         elif command == 0x6f:
             self._erase_all_unprotected()
         else:
